package com.flyer.foster.util;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.flyer.exception.BusinessException;
import com.flyer.foster.consts.RedisKeyConst;
import com.flyer.foster.consts.WechatConst;
import com.flyer.foster.converter.WxMappingJackson2HttpMessageConverter;
import com.flyer.foster.pojo.Code2SessionResp;
import com.flyer.util.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import javax.annotation.PostConstruct;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;

/**
 * 微信小程序api工具类
 */
@Component
@Slf4j
public class WeChatApiUtil {
    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private RedisUtil redisUtil;

    @PostConstruct
    public void initRestTemplate() {
        // 解决调用微信服务返回信息无法反序列化
        restTemplate.getMessageConverters().add(new WxMappingJackson2HttpMessageConverter());
    }

    /**
     * 根据code（临时调用凭证），请求微信api获取用户unionid
     *
     * @param code      前端调用微信登录接口生成的临时凭证
     * @param appId     微信小程序的appId
     * @param appSecret 小程序appId对应的秘钥
     */
    public Code2SessionResp getCode2Session(String code, String appId, String appSecret) {
        String url = "https://api.weixin.qq.com/sns/jscode2session?appid={appid}&secret={secret}&js_code={code}&grant_type={authorization_code}";
        HashMap<String, String> map = new HashMap<>();
        map.put("appid", appId);
        map.put("secret", appSecret);
        map.put("code", code);
        map.put("authorization_code", "authorization_code");
        Code2SessionResp resp = new Code2SessionResp();
        try {
            resp = restTemplate.getForObject(url, Code2SessionResp.class, map);
        } catch (RestClientException e) {
            throw new BusinessException("调用微信服务错误");
        }
        return resp;
    }

    /**
     * 获取微信调用凭证
     *
     * @return
     */
    public JSONObject getAccessToken1(String appId, String secret) {
        String url = "https://api.weixin.qq.com/cgi-bin/token?grant_type={grantType}&appid={appId}&secret={secret}";
        HashMap<String, String> map = new HashMap<>();
        map.put("grantType", "client_credential");
        map.put("appId", appId);
        map.put("secret", secret);

        JSONObject res;
        try {
            res = restTemplate.getForObject(url, JSONObject.class, map);
        } catch (RestClientException e) {
            throw new BusinessException("调用微信服务错误");
        }
        return res;
    }

    public String getAccessToken(String appId, String secret) {
        // 从缓存中获取accessToken
        Object accessTokenCache = redisUtil.get(RedisKeyConst.WECHAT_ACCESS_TOKEN_KEY + appId);
        if (accessTokenCache == null) {
            String accessToken;
            String url = "https://api.weixin.qq.com/cgi-bin/token?grant_type={grantType}&appid={appId}&secret={secret}";
            HashMap<String, String> map = new HashMap<>();
            map.put("grantType", "client_credential");
            map.put("appId", appId);
            map.put("secret", secret);
            JSONObject res;
            try {
                res = restTemplate.getForObject(url, JSONObject.class, map);
                // token失效时间
                Integer expires = res.getInteger("expires_in");
                long cacheExpires = expires - 100;
                accessToken = res.getString("access_token");
                redisUtil.set(RedisKeyConst.WECHAT_ACCESS_TOKEN_KEY + appId, accessToken, cacheExpires);
            } catch (RestClientException e) {
                throw new BusinessException("调用微信服务错误");
            }
            return accessToken;
        } else {
            return accessTokenCache.toString();
        }
    }

    //    public String getPhoneNumber(String appId, String appSecret, String code) {
//        String accessToken = this.getAccessToken(appId, appSecret);
//        String tel = "";
//        String url = StrUtil.format("https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token={}", accessToken);
//        try {
//            HttpHeaders headers = new HttpHeaders();
//            headers.setContentType(MediaType.APPLICATION_JSON);
//            HashMap<String, String> bodyParams = new HashMap<>();
//            bodyParams.put("code", code);
//            // 用HttpEntity封装整个请求报文
//            HttpEntity<Map<String, String>> httpEntity = new HttpEntity<>(bodyParams, headers);
//            JSONObject res = restTemplate.postForObject(url, httpEntity, JSONObject.class);
//            if (res != null) {
//                tel = res.getJSONObject("phone_info").getString("phoneNumber");
//            }
//        } catch (RestClientException e) {
//            throw new BusinessException("调用微信服务错误");
//        }
//        return tel;
//    }

    /**
     * 给订阅用户发送消息
     *
     * @param accessToken
     * @param openId
     * @param templateId  消息模板id
     * @return
     */
    public Map<String, Object> subscribeMessage(String accessToken, String openId, String templateId) {
        // 首先获取微信调用凭证
        if (null == accessToken) {
            throw new BusinessException("获取微信调用凭证失败");
        }
        String url = "https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token=" + accessToken;

        HashMap<String, Object> param = new HashMap<>();
        param.put("template_id", templateId);
        param.put("touser", openId);
        param.put("page", "pages/subPages/login/index");
        param.put("miniprogram_state", "trial");
        JSONObject data = new JSONObject();
        // 留言内容
        data.put("thing2", setVal("value", "test12399"));
        // 留言人
        data.put("name1", setVal("value", "守护一棵树"));
        // 留言时间
        data.put("time3", setVal("value", LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"))));

        param.put("data", data);
        try {
            // 微信调用返回结果
            JSONObject res = restTemplate.postForObject(url, param, JSONObject.class);
            if (res != null && res.get("errcode").equals(0)) {
                return (Map<String, Object>) res.get("result");
            } else {
                log.error(res.get("errmsg").toString());
                return null;
            }
        } catch (RestClientException e) {
            throw new BusinessException("http客户端请求错误");
        }
    }

    private JSONObject setVal(String key, String value) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(key, value);
        return jsonObject;
    }
}
